//---------------------------------------------------------------------------
//	Greenplum Database
//	Copyright (C) 2012 EMC Corp.
//
//	@filename:
//		CParseHandlerWindowKey.h
//
//	@doc:
//		SAX parse handler class for parsing the window key
//---------------------------------------------------------------------------

#ifndef GPDXL_CParseHandlerWindowKey_H
#define GPDXL_CParseHandlerWindowKey_H

#include "gpos/base.h"
#include "naucrates/dxl/operators/CDXLWindowKey.h"
#include "naucrates/dxl/parser/CParseHandlerBase.h"
#include "naucrates/dxl/xml/dxltokens.h"

namespace gpdxl
{
	using namespace gpos;

	XERCES_CPP_NAMESPACE_USE

	//---------------------------------------------------------------------------
	//	@class:
	//		CParseHandlerWindowKey
	//
	//	@doc:
	//		SAX parse handler class for parsing the window key
	//
	//---------------------------------------------------------------------------
	class CParseHandlerWindowKey : public CParseHandlerBase
	{
		private:

			// window key generated by the parser
			CDXLWindowKey *m_pdxlwk;

			// private copy ctor
			CParseHandlerWindowKey(const CParseHandlerWindowKey&);

			// process the start of an element
			void StartElement
					(
					const XMLCh* const xmlszUri, 		// URI of element's namespace
 					const XMLCh* const xmlszLocalname,	// local part of element's name
					const XMLCh* const xmlszQname,		// element's qname
					const Attributes& attr				// element's attributes
					);

			// process the end of an element
			void EndElement
					(
					const XMLCh* const xmlszUri, 		// URI of element's namespace
					const XMLCh* const xmlszLocalname,	// local part of element's name
					const XMLCh* const xmlszQname		// element's qname
					);

		public:
			// ctor/dtor
			CParseHandlerWindowKey
				(
				IMemoryPool *pmp,
				CParseHandlerManager *pphm,
				CParseHandlerBase *pphRoot
				);

			// window key generated by the parse handler
			CDXLWindowKey *Pdxlwk() const
			{
				return m_pdxlwk;
			}
	};
}

#endif // !GPDXL_CParseHandlerWindowKey_H

// EOF
